﻿using System;
using System.Collections;
using Alachisoft.NCache.Web.Caching;

// ===============================================================================
// Alachisoft (R) NCache Sample Code.
// NCache Groups sample class
// ===============================================================================
// Copyright © Alachisoft.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
// ===============================================================================


namespace GroupsAndTags
{
	public class Groups
	{

		public static void RunGroupsDemo()
		{
			try
			{
				Console.WriteLine();

				Cache cache = NCache.InitializeCache("mypartitionedcache");
				cache.Clear();

				//Adding item in same group
				//Group can be done at two levels 
				//Groups and Subgroups.          

				cache.Add("Product:CellularPhoneHTC", "HTCPhone", "Electronics", "Mobiles");
				cache.Add("Product:CellularPhoneNokia", "NokiaPhone", "Electronics", "Mobiles");
				cache.Add("Product:CellularPhoneSamsung", "SamsungPhone", "Electronics", "Mobiles");
				cache.Add("Product:ProductLaptopAcer", "AcerLaptop", "Electronics", "Laptops");
				cache.Add("Product:ProductLaptopHP", "HPLaptop", "Electronics", "Laptops");
				cache.Add("Product:ProductLaptopDell", "DellLaptop", "Electronics", "Laptops");
				cache.Add("Product:ElectronicsHairDryer", "HairDryer", "Electronics", "SmallElectronics");
				cache.Add("Product:ElectronicsVaccumCleaner", "VaccumCleaner", "Electronics", "SmallElectronics");
				cache.Add("Product:ElectronicsIron", "Iron", "Electronics", "SmallElectronics");

				// Getting group data
				IDictionary items = cache.GetGroupData("Electronics", null); // Will return nine items since no subgroup is defined;
				if (items.Count > 0)
				{
					Console.WriteLine("Item count: " + items.Count);
					Console.WriteLine("Following Products are found in group 'Electronics'");
					IEnumerator itor = items.Values.GetEnumerator();
					while (itor.MoveNext())
					{
						Console.WriteLine(itor.Current.ToString());
					}
					Console.WriteLine();
				}

				items = cache.GetGroupData("Electronics", "Mobiles"); // Will return thre items under the subgroup Mobiles
				if (items.Count > 0)
				{
					Console.WriteLine("Item count: " + items.Count);
					Console.WriteLine("Following Products are found in group 'Electronics' and Subgroup 'Mobiles'");
					IEnumerator itor = items.Values.GetEnumerator();
					while (itor.MoveNext())
					{

						Console.WriteLine(itor.Current.ToString());
					}
					Console.WriteLine();
				}

				//getGroupKeys is yet another function to retrive group data.
				//It however requires multiple iterations to retrive actual data
				//1) To get List of Keys and 2) TO get items for the return List of Keys

				//Updating items in groups
				cache.Insert("Product:ElectronicsIron", "PanaSonicIron", "Electronics", "SmallElectronics"); //Item is updated at the specified group


				//Removing group data
				Console.WriteLine("Item count: " + cache.Count); // Itemcount = 9
				cache.RemoveGroupData("Electronics", "Mobiles"); // Will remove 3 items from cache based on subgroup Mobiles

				Console.WriteLine("Item count: " + cache.Count); // Itemcount = 6
				cache.RemoveGroupData("Electronics", null); // Will remove all items from cache based on group Electronics

				Console.WriteLine("Item count: " + cache.Count); // Itemcount = 0

				Console.WriteLine();

				//Must dispose cache
				cache.Dispose();

			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
			}
		}
	}

}